# Correction Steps
CORRECTION_STEP_HEADING_COLOR = '#DDEBF7'  # Soft Light Blue for heading
CORRECTION_STEP_BOX_COLOR = '#B7D7F7'  # Slightly darker Light Blue for box

# Corruption Steps
CORRUPTION_STEP_HEADING_COLOR = '#FCE4D6'  # Soft Peach for heading
CORRUPTION_STEP_BOX_COLOR = '#F9C8B3'  # Slightly darker Peach for box

# Simulation Steps
SIMULATION_STEP_HEADING_COLOR = '#E2EFDA'  # Soft Light Green for heading
SIMULATION_STEP_BOX_COLOR = '#C1DFB5'  # Slightly darker Light Green for box

# Validation Steps
VALIDATION_STEP_HEADING_COLOR = '#FFF2CC'  # Soft Light Yellow for heading
VALIDATION_STEP_BOX_COLOR = '#FFE29F'  # Slightly darker Light Yellow for box

# Distillation Steps
DISTILLATION_STEP_HEADING_COLOR = '#E4D6F1'  # Soft Lavender for heading
DISTILLATION_STEP_BOX_COLOR = '#D1BEEA'  # Slightly darker Lavender for box
